import tkinter as tk

from .controls import Controls
from .display import Display
from .play_pause import PlayPause
from .tapper import Tapper
from ..model import Model


class TopView(tk.Frame):
    def __init__(self, master: tk.Tk, model: Model) -> None:
        super().__init__(master=master)

        top_frame = tk.Frame(master=self)
        left_frame = tk.Frame(master=top_frame)
        display = Display(master=left_frame, model=model)
        display.pack(side=tk.TOP)

        play_pause = PlayPause(master=left_frame, model=model)
        play_pause.pack(side=tk.BOTTOM)

        left_frame.pack(side=tk.LEFT)

        controls = Controls(master=top_frame, model=model)
        controls.pack(side=tk.RIGHT)
        top_frame.pack(side=tk.TOP)

        tapper = Tapper(master=self, model=model)
        tapper.pack(side=tk.BOTTOM)

        self.pack()



import tkinter as tk

from ..controller import ControlsController
from ..model import Model


class Controls(tk.Frame):
    def __init__(
        self,
        master: tk.Frame,
        model: Model,
    ) -> None:
        super().__init__(master=master)

        controller = ControlsController(model)

        up_button = tk.Button(master=self, text='+', width=2)
        up_button.bind('<Button 1>', lambda event: controller.up_action())
        up_button.pack()

        down_button = tk.Button(master=self, text='-', width=2)
        down_button.bind('<Button 1>', lambda event: controller.down_action())
        down_button.pack()



import tkinter as tk

from ..model import Model


class Display(tk.Frame):
    def __init__(self, master: tk.Frame, model: Model) -> None:
        super().__init__(master=master, height=2)

        self.bpm = tk.Label(master=self, text=str(model.bpm))
        self.bpm.pack(side=tk.LEFT)

        unit = tk.Label(master=self, text='bpm')
        unit.pack(side=tk.RIGHT)

        model.register_for_bpm(self)

    def update_(self, value: float) -> None:
        self.bpm['text'] = str(int(value))



import pathlib

from setuptools import find_packages, setup


here = pathlib.Path(__file__).parent

readme = (here / 'README.md').read_text()

packages = find_packages(exclude=('tests',))

setup(
    name='etronome',
    version='0.1.0',
    description='Keep track of a rythm',
    long_description=readme,
    long_description_content_type='text/markdown',
    url='https://github.com/wpbindt/metronome',
    author='Wessel Bindt',
    author_email='wesselbindt@gmail.com',
    license='GNU',
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python :: 3.8'
    ],
    packages=packages,
    include_package_data=True,
)




import tkinter as tk

from ..beat_counter import BeatCounter
from ..controller import TapperController
from ..model import Model


class Tapper(tk.Frame):
    def __init__(
        self,
        master: tk.Frame,
        model: Model
    ) -> None:
        super().__init__(master=master)

        beat_counter = BeatCounter(model)
        controller = TapperController(beat_counter)

        button = tk.Button(master=self, text='tap me', width=10)
        button.bind('<Button 1>', lambda event: controller.button_action())
        button.pack()



import datetime

from .model import Model


class BeatCounter:
    TIMEOUT = 1.5

    def __init__(self, model: Model) -> None:
        self._model = model
        self._tap_times: list[datetime.datetime] = []

    def tap(self) -> None:
        tap_time = datetime.datetime.now()
        if len(self._tap_times) == 0:
            self._tap_times.append(tap_time)
            return

        time_since_last_tap = (tap_time - self._tap_times[-1]).total_seconds()
        if time_since_last_tap > BeatCounter.TIMEOUT:
            self._tap_times = [tap_time]
        else:
            self._tap_times.append(tap_time)
            self._model.bpm = self._bpm

    @property
    def _bpm(self) -> float:
        total_time = (self._tap_times[-1] - self._tap_times[0]).total_seconds()
        return 60 * (len(self._tap_times) - 1) / total_time



import tkinter as tk

from ..controller import PlayPauseController
from ..model import Model


class PlayPause(tk.Frame):
    def __init__(
        self,
        master: tk.Frame,
        model: Model
    ) -> None:
        super().__init__(master=master)

        controller = PlayPauseController(model)

        self.button = tk.Button(master=self, text='play')
        self.button.bind(
            '<Button 1>',
            lambda event: controller.button_action()
        )
        self.button.pack()

        model.register_for_is_playing(self)
        self.update_(model.is_playing)

    def update_(self, value: bool) -> None:
        self.button['text'] = 'pause' if value else 'play'
